%% Copyright (c) 2016,2017 Brook Milligan.
%%
%% This file is part of the GraphModel Library.
%%
%%     The GraphModel Library is free software: you can redistribute
%%     it and/or modify it under the terms of the GNU Affero General
%%     Public License as published by the Free Software Foundation,
%%     either version 3 of the License, or (at your option) any later
%%     version.
%%
%%     The GraphModel Library is distributed in the hope that it will
%%     be useful, but WITHOUT ANY WARRANTY; without even the implied
%%     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
%%     PURPOSE.  See the GNU Affero General Public License for more
%%     details.
%%
%%     You should have received a copy of the GNU Affero General
%%     Public License along with the GraphModel Library.  If not, see
%%     <http://www.gnu.org/licenses/>.
%%

\mainpage Graph Model Library


\tableofcontents


\section related_pages Related pages:

- \subpage page_simplified_class_hierarchy
- \subpage page_building_models


\section notes Random notes

Main classes, all of which are proto expressions in the simple_domain and
inherit from node_base<T> where T is the value type; all are default constructible
- rv_terminal<RV>: random variable parameterized on the type-specific implementation class RV
  and the value type T
  - general properties
    - the variate is always a terminal; i.e., it is not an expression tree.  if it is
      initialized with an expression tree, that tree is evaluated to initialize the terminal
    - this is the only object (excepting rv_pointer which delegates everything) that can 
      participate in MCMC updates
    - this is the only object (likewise excepting rv_pointer) that can be a proposal distribution
  - type trait categories
    - a terminal
    - an rv
  - constructible from:
    - an arbitrary argument forwarded to the constructor of the implementation class RV
      via proto::make()
    - arbitrary arguments (>1) forwarded to initialize (not construct) the
      implementation class RV
- rv_pointer<T,U,...>: a pointer to node_base<U> with a value type of T
  - type trait categories
    - a terminal
    - an rv
  - constructible from:
    - self, i.e., copy construction
    - rv_pointer<U,V>: cloned
    - rv_pointer<V,U>: pointer copy
    - rv_terminal<RV>: cloned if U is the value type of RV
    - simple_expression<Expr>: cloned if U is the value type of Expr
- simple_expression<Expr>: an encapsulation of a general proto expression
  - general properties
    - this represents an expression tree involving random variables and therefore cannot
      participate in MCMC updates (there is no value to update)
  - type trait categories
    - not a terminal
    - not an rv
  - constructible from proto expressions

Specializations of rv_terminal<RV> with different implementation
classes RV represent a range of random variables
- literal terminals created by lit()
- random variables characterized by specific distributions

All random variables provide the following functionality,
accessible through the base class node_base<T> (and hence via the
pointers rv_pointer)
- value_type value() const
- value_type sample() const
- probability_type pdf() const
- probability_type mcmc_pdf() const [?]
- void proposal()
- void accept()
- void reject()
- address_type address() const
- identifier_type id() const
- tag_type tag() const [?]
- expression tree evaluation

[ DELETE ] Need rules for combining random variables
- constants: dirac delta distributions
- D: some other distribution
- constant OP constant -> a new constant determined by OP
- D + constant -> implemented for each RV to shift mean only
- D op constant -> some new distribution
  - perhaps possible in general (i.e., without simulation)
    only for conjugate distributions/operations

Details of type-specific implementation class RV
- core type-specific data members
  - parameter data member(s): general expression tree(s)
    held as rv_pointer objects, possibly including *this
- core type-specific implementation functions
  - pdf(variate|parameter): type-specific pdf function
    - if constant: result = 1 if variate within domain, 0 otherwise
    - otherwise: result = type-specific pdf(variate|parameter) if variate within domain, 0 otherwise
  - sample(variate,parameter): type-specific sampling
    - if constant: result = variate
    - otherwise: result = type-specific sample(parameter)
- pdf(pdf_context) -> (Pr(current),Pr(proposal))
  - (current,proposal) = pdf_context.variate()
  - (parameter,ignored) = parameter(parameter_context(pdf_context))
  - delegate to RV::pdf(current|parameter) and RV::pdf(proposal|parameter)
- sample(sample_context) -> sample
  - (current,ignored) = sample_context.variate()
  - (parameter,ignored) = parameter(parameter_context(sample_context))
  - delegate to RV::sample(variate,parameter)
- parameter(parameter_context) -> (current,proposal) [ private ]
  - parameter_expression_tree.value(value_context(parameter_context))

Details of rv_terminal<RV> data members
- identifier
- address
- type-specific RV: see above; stored as proto::term<RV>
- values (= variate): shared pointer to a pair of type T
  - current variate
  - proposal
- proposer: (an rv_pointer to) an rv_terminal<RV>
- pdf: (a pointer to) a shared implicit multiplicative expression tree; must not include *this

Details of rv_terminal<RV> member functions
- id() -> user-provided identifier
- address() -> this
- variate() = value() [ distinct names for different contexts; user-facing versus
  variate/parameter distinction ]
- value() -> (current,proposal)
- value(value_context) -> value(); i.e., ignore context
- sample() = RV::sample(sample_context(variate()))
- sample(sample_context) = sample(); i.e., ignore context
- pdf() = product of terms -> (Pr(current),Pr(proposal))
- pdf(pdf_context) = (pdf(pdf_context.current()),pdf(pdf_context.proposed()))
- pdf_insert() = pdf_insert(pdf_insert_context(*this))
- pdf_insert(pdf_insert_context): add a term to the implicit multiplicative expression
  tree for pdf(); note: must traverse parameters in RV and must avoid inserting *this

All random variables hold both variates and parameters as
rv_pointers and allow replacement of them by new expression trees
as appropriate.  In addition, random variables hold pointers to
proposal distributions (which are other random variables) and these
may be replaced as appropriate.  All random variables may be held in
a symbol table containing appropriate rv_pointers.

Implementation of MCMC:
- requirements for acceptance/rejection
  - proposal = sample(proposer parameters)
  - Pr(proposal|current,proposer parameters)
  - Pr(current|proposal,proposer parameters)
  - Pr(current|distribution parameters)
  - Pr(proposal|distribution parameters)
- in presence of multiple threads
  - (proposal,2 transition probabilities) = f(current,proposer parameters)
  - (Pr(current),Pr(proposal)) = f(proposal,current,distribution parameters)
- proposal
  - create a proposal from the distribution; store in proposal
  - current/proposal pdf: pdf(variate) = Pr(variate|parameters)
  - transition probability: pdf(variate) = Pr(variate|parameters)
- accept/reject
- proposal distributions:
  - sample(): called to generate proposal
  - pdf(variate1,variate2) = (Pr(variate1|parameters),Pr(variate2|parameters))


\section PointersHandles Pointers and handles

- Distributions (derived from rv_base)
  - only allocated on the heap
  - constructed by factory functions
  - initially owned by a unique_ptr
  - transferred to a shared_ptr with unique ownership semantics
    - shared_ptr needed to allow obtaining weak_ptrs
    - shared_ptr held by the owning random variable
  - target of weak_ptrs to execute pdf(), sample(), etc.
  - hold a (raw?) pointer to the corresponding random variable
  - holds pointers to its parameters, if any
- Random variables (rv_terminal)
  - stack or allocated on heap
  - owner of (unique) shared_ptr to its distribution
  - target of the (raw?) pointer held by its distribution
  - potentially target of a random variable handle
  - constructed by a factory function
- Random variable handles (rv_pointer)
  - owner of a unique (not necessarily owning) pointer to a random variable
- Dirac delta distribution (derived from rv_base)
  - above applies
  - holds a pointer to its base class?
