/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef ASYNCHRONOUS_SIGNALS_MEAN_COMBINER_HPP
#define ASYNCHRONOUS_SIGNALS_MEAN_COMBINER_HPP 1

#include <traits/real.hpp>
#include <boost/log/sources/severity_logger.hpp>
#include <boost/log/trivial.hpp>
#include <logging/channel_log.hpp>
#include <logging/global_log.hpp>

namespace asynchronous
{

  namespace signals
  {

    template < typename T
	       , typename Count = long
	       >
    class mean_combiner
    {
      using self = mean_combiner;
      using real_type = ::traits::real_t<T>;
      using log_type = logging::channel_log;

    public:
      auto result () const
      {
	using namespace boost::log::trivial;
	auto result = count_ ? static_cast<real_type>(sum_) / count_ : sum_;
	GLOBAL_LOG(log_,trace) << "result()const: sum_= " << sum_ << ", count_=" << count_ << ", result=" << result;
	return result;
      }

      void merge (self const& other)
      {
	using namespace boost::log::trivial;
	sum_ += other.sum_;
	count_ += other.count_;
	GLOBAL_LOG(log_,trace) << "result()const: sum_=" << sum_ << ", count_=" << count_;
      }

      void operator () (T t)
      {
	using namespace boost::log::trivial;
	sum_ += t;
	++count_;
	GLOBAL_LOG(log_,trace) << "operator()(T): t=" << t << ", sum_=" << sum_ << ", count_=" << count_;
      }

    private:
      T sum_ = 0;
      Count count_ = 0;
      mutable log_type log_ = "asynchronous::signals::mean_combiner";
    };

  } // namespace signals

} // namespace asynchronous

#endif
