/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_GRAPH_VERTEX_LIST_GRAPH_BASE_HPP
#define GRAPH_MODEL_GRAPH_VERTEX_LIST_GRAPH_BASE_HPP 1

#include <set>
#include <boost/graph/graph_traits.hpp>
#include <graph_model/graph/bfs_visitor.hpp>
#include <graph_model/graph/edge_inserter.hpp>
#include <graph_model/graph/graph_base.hpp>
#include <graph_model/graph/vertex_inserter.hpp>
#include <graph_model/traits/vertex_descriptor.hpp>
#include <graph_model/traits/vertex_iterator.hpp>

namespace graph_model
{

  namespace graph
  {

    template < typename Derived, typename Tag, typename Base = graph_base >
    class vertex_list_graph_base
      : public Base
    {
      using self = vertex_list_graph_base;

      using vertex_descriptor = traits::vertex_descriptor_t<Tag>;
      using vertex_set_type = std::set<vertex_descriptor>;

      template < typename T >
      friend class vertex_inserter;

    public:
      vertex_list_graph_base () = default;
      
      template < typename Graph >
      vertex_list_graph_base (Graph const& g, typename boost::graph_traits<Graph>::vertex_descriptor source)
	: Base(g,source)
      { derived().add_connected_component(g,source); }

      auto num_vertices () const { return vertices_.size(); }
      auto vertices () const
      {
	using vertex_iterator = traits::vertex_iterator_t<Tag>;
	return std::make_pair(vertex_iterator(vertices_),vertex_iterator());
      }

    public:			// XXX - for CRTP
      auto bfs_visitor() { return make_bfs_visitor(make_vertex_inserter(*this),make_edge_inserter()); }

    private:
      void insert_vertex_descriptor (vertex_descriptor v)
      { vertices_.insert(v); }

    private:
      Derived& derived () { return static_cast<Derived&>(*this); }
      Derived const& derived () const { return static_cast<Derived const&>(*this); }

    private:
      vertex_set_type vertices_;
    };

    template < typename Derived, typename Tag, typename Base >
    auto num_vertices (vertex_list_graph_base<Derived,Tag,Base> const& g) { return g.num_vertices(); }

    template < typename Derived, typename Tag, typename Base >
    auto vertices (vertex_list_graph_base<Derived,Tag,Base> const& g) { return g.vertices(); }

  } // namespace graph

} // namespace graph_model

#endif
