/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_UTILITY_UNIQUE_FUNCTION_COLLECTION_HPP
#define GRAPH_MODEL_UTILITY_UNIQUE_FUNCTION_COLLECTION_HPP 1

#include <algorithm>
#include <graph_model/utility/function_collection.hpp>

namespace graph_model
{

  namespace utility
  {

    /// \brief Unique collection of function objects
    /// \tparam Signature The function signature
    ///
    /// This class maintains a set of unique functions.  It is built
    /// upon \c graph_model::utility::function_collection, which is a
    /// (temporary) replacement for
    /// boost::poly_collection::function_collection.
    ///
    template < typename Signature >
    class unique_function_collection
      : public function_collection<Signature>
    {
      using base = function_collection<Signature>;

    public:
      template < typename F >
      auto insert (F&& f) { return base::insert_unique(std::forward<F>(f)); }

      template < typename Iterator, typename F >
      auto insert (Iterator, F&& f) { return insert(std::forward<F>(f)); }
    };

  } // namespace utility

} // namespace graph_model

#endif
