/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_CDF_BASE_HPP
#define GRAPH_MODEL_VERTEX_CDF_BASE_HPP 1

#include <graph_model/vertex/type/probability_base.hpp>
#include <graph_model/vertex/type/variate_base.hpp>

namespace graph_model
{

  namespace vertex
  {

    template < typename Variate, typename Probability >
    class cdf_base
      : public virtual type::probability_base<Probability>
      , public virtual type::variate_base<Variate>
    {
      using probability_base = type::probability_base<Probability>;
      using variate_base = type::variate_base<Variate>;

    public:
      auto cdf () const { return cdf_(); }
      auto cdf (typename variate_base::variate_type variate) const { return cdf_(variate); }

    private:
      virtual typename probability_base::probability_type cdf_ () const = 0;
      virtual typename probability_base::probability_type cdf_ (typename variate_base::variate_type) const = 0;
    };

    template < typename Probability >
    class cdf_base< void,Probability >
      : public virtual type::probability_base<Probability>
    {
      using base = type::probability_base<Probability>;

    public:
      auto cdf () const { return cdf_(); }

    private:
      virtual typename base::probability_type cdf_ () const = 0;
    };

  } // namespace vertex

} // namespace graph_model

#endif
