/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_FACTOR_HPP
#define GRAPH_MODEL_VERTEX_FACTOR_HPP 1

#include <graph_model/vertex/factor_fwd.hpp>
#include <boost/yap/algorithm.hpp>
#include <graph_model/vertex/expression_terminal.hpp>
#include <graph_model/vertex/factor_base.hpp>
#include <graph_model/vertex/factor_impl_base.hpp>
#include <graph_model/vertex/strategy/basic_factor.hpp>
#include <graph_model/vertex/strategy_owner_base.hpp>
#include <graph_model/vertex/tags.hpp>
#include <graph_model/vertex/vertex_expression.hpp>
#include <mpl/enable_constructor.hpp>

namespace graph_model
{

  namespace vertex
  {

    /// \brief A concrete factor class
    ///
    /// A factor delegates its \c eval() and \c pdf() member functions
    /// to a strategy class.
    ///
    template < typename Probability >
    class factor
      : public factor_impl_base<factor<Probability>,tag::factor,factor_base<Probability>>
      , public strategy_owner_base<factor<Probability>,strategy::basic_factor_factory<Probability>>
    {};

    template < typename ... T, typename ... Args >
    auto transform_expression (expression_terminal<factor<T...>> expr, Args&& ... args)
    { return evaluate_terminal(expr,std::forward<Args>(args)...); }
					   
    template < typename ... T, typename ... Args >
    auto transform_expression (expression_terminal<factor<T...>&> expr, Args&& ... args)
    { return evaluate_terminal(expr,std::forward<Args>(args)...); }

    template < typename Probability, typename ... Args >
    auto make_factor (Args&& ... args)
    {
      using factor_type = factor<Probability>;
      factor_type factor(std::forward<Args>(args)...);
      return factor;
    }

  } // namespace vertex

} // namespace graph_model

#endif
