/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_SCALAR_FACTOR_BASE_HPP
#define GRAPH_MODEL_VERTEX_SCALAR_FACTOR_BASE_HPP 1

#include <graph_model/vertex/cdf_base.hpp>
#include <graph_model/vertex/factor_base.hpp>
#include <mpl/enable_constructor.hpp>

namespace graph_model
{

  namespace vertex
  {

    template < typename Variate, typename Probability >
    class scalar_factor_base
      : public factor_base<Probability>
      , public cdf_base<Variate,Probability>
    {
      using self = scalar_factor_base;
      using base = factor_base<Probability>;

    public:
      scalar_factor_base () = default;
      scalar_factor_base (scalar_factor_base&&) = default;
      scalar_factor_base& operator = (scalar_factor_base&&) = default;
      virtual ~scalar_factor_base () = default;

      template < typename ... Args
		 , typename = mpl::enable_constructor_t<self,Args...>
		 >
      explicit scalar_factor_base (Args&& ... args)
	: base(std::forward<Args>(args)...)
      {}
    };

  } // namespace vertex

} // namespace graph_model

#endif
