/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_SUMMARIZED_VARIATE_IMPL_BASE_HPP
#define GRAPH_MODEL_VERTEX_SUMMARIZED_VARIATE_IMPL_BASE_HPP 1

#include <graph_model/accumulators/accumulator_set.hpp>
#include <use_default.hpp>

namespace graph_model
{

  namespace vertex
  {

    template < typename Derived
	       , typename Variate
	       , typename Stats = use_default<> // XXX - unused
	       , typename Base = void
	       >
    struct summarized_variate_impl_base
      : Base
    {
      using base = Base;
      using accumulator_set_type = accumulators::accumulator_set_t<Variate>; // XXX - use Stats

    public:
      auto& accumulator () const { return accumulator_; }

    private:
      void updated_variate_ () override { accumulator_(derived().current_variate()); }

    private:
      auto& derived () { return static_cast<Derived&>(*this); }
      auto& derived () const { return static_cast<Derived const&>(*this); }

    private:
      accumulator_set_type accumulator_;
    };

  } // namespace vertex

} // namespace graph_model

#endif
