/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_VERTEX_DESCRIPTOR_BASE_HPP
#define GRAPH_MODEL_VERTEX_VERTEX_DESCRIPTOR_BASE_HPP 1

#include <graph_model/vertex/vertex_descriptor_base_fwd.hpp>
#include <graph_model/vertex/identifier_base.hpp>
#include <graph_model/vertex/vertex_category_base.hpp>
#include <graph_model/vertex/vertex_descriptor.hpp>
#include <graph_model/vertex/vertex_observer_collection_base.hpp>
#include <mpl/enable_constructor.hpp>

namespace graph_model
{

  namespace vertex
  {

    class vertex_descriptor_base
      : public vertex_category_base<>
      , public identifier_base<>
      , public vertex_observer_collection_base<vertex_descriptor_base>
    {
      using self = vertex_descriptor_base;
      using base = vertex_category_base<>;

    public:
      vertex_descriptor_base () = default;
      vertex_descriptor_base (self&&) = default;
      self& operator = (self&&) = default;
      virtual ~vertex_descriptor_base () = default;

      template < typename ... Args
		 , typename = mpl::enable_constructor_t<self,Args...>
		 >
      explicit vertex_descriptor_base (Args&& ... args)
	: base(std::forward<Args>(args)...)
      {}

      auto vertex_descriptor () const { return make_vertex_descriptor(*this); }
    };

  } // namespace vertex

} // namespace graph_model

#endif
