/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_VERTEX_IMPL_BASE_HPP
#define GRAPH_MODEL_VERTEX_VERTEX_IMPL_BASE_HPP 1

#include <graph_model/vertex/adjacency_base.hpp>
#include <graph_model/vertex/category_base.hpp>
#include <graph_model/vertex/identifier_impl_base.hpp>
#include <graph_model/vertex/tags.hpp>
#include <graph_model/vertex/vertex_base.hpp>

// #include <graph_model/vertex/adjacency_owner_base.hpp>
// #include <graph_model/vertex/move_base.hpp>

namespace graph_model
{

  namespace vertex
  {

    /// \brief Base class for a graph model vertex
    ///
    /// This base class gathers together all the vertex functionality
    /// contained in distinct base classes.
    ///
    template < typename Derived, typename Category, typename Base >
    struct vertex_impl_base
      : identifier_impl_base<Derived
      , category_base<Category
      , Base
      > >
      , adjacency_base<Derived>
      // , adjacency_owner_base<Derived>
      // , move_base<Derived>
    {};

  } // namespace vertex

} // namespace graph_model

#endif
