/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef SAMPLE_SPACE_MATH_MULTINOMIAL_PDF_RATIO_HPP
#define SAMPLE_SPACE_MATH_MULTINOMIAL_PDF_RATIO_HPP 1

namespace sample_space
{
  
  namespace math
  {

    template < typename MultinomialPDF >
    class multinomial_pdf_ratio
    {
      using self = multinomial_pdf_ratio;
      using value_type = typename MultinomialPDF::value_type;

    public:
      multinomial_pdf_ratio () = default;
      
      multinomial_pdf_ratio (MultinomialPDF numerator, MultinomialPDF denominator)
	: numerator_(numerator)
	, denominator_(denominator)
      {}

      operator value_type () const { return value(); }

      // XXX - make this more efficient
      value_type value () const
      { return numerator_.value() / denominator_.value(); }

    private:
      MultinomialPDF numerator_;
      MultinomialPDF denominator_;
    };
    
    template < typename MultinomialPDF >
    auto make_multinomial_pdf_ratio (MultinomialPDF numerator,
				     MultinomialPDF denominator)
    {
      using multinomial_pdf_ratio_type = multinomial_pdf_ratio<MultinomialPDF>;
      return multinomial_pdf_ratio_type(numerator,denominator);
    }

  } // namespace math

} // namespace sample_space

#endif
