/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef SAMPLE_SPACE_MATH_NORM_1_HPP
#define SAMPLE_SPACE_MATH_NORM_1_HPP 1

#include <cmath>
#include <boost/optional.hpp>
#include <boost/range/algorithm/for_each.hpp>
#include <traits/mapped_type.hpp>
#include <traits/value.hpp>
#include <type_traits/is_map.hpp>

namespace sample_space
{
  
  namespace math
  {

    namespace detail
    {

      template < typename Range
		 , typename = type_traits::is_map_t<Range>
		 >
      class norm_1;

      template < typename Range >
      class norm_1< Range,std::false_type >
      {
	using value_type = traits::value_t<Range>;

      public:
	static value_type apply (Range range)
	{
	  using std::begin;
	  using std::endl;
	  return apply(begin(range),end(range));
	}

	template < typename Iterator >
      	static value_type apply (Iterator begin, Iterator end)
	{
	  using result_type = boost::optional<value_type>;
	  using std::abs;
	  result_type result;
	  while (begin != end)
	    {
	      auto term = abs(*begin);
	      if (result)
		*result += term;
	      else
		result = term;
	      ++begin;
	    }
	  return result ? *result : 0;
	}
      };
      
      template < typename Range >
      class norm_1< Range,std::true_type >
      {
	using value_type = traits::mapped_type_t<Range>;

      public:
	static value_type apply (Range range)
	{
	  using std::begin;
	  using std::endl;
	  return apply(begin(range),end(range));
	}

	template < typename Iterator >
	static value_type apply (Iterator begin, Iterator end)
	{
	  using result_type = boost::optional<value_type>;
	  using std::abs;
	  using boost::for_each;
	  result_type result;
	  while (begin != end)
	    {
	      auto term = abs(begin->second);
	      if (result)
		*result += term;
	      else
		result = term;
	      ++begin;
	    }
	  return result ? *result : 0;
	}
      };
      
    } // namespace detail
    
    template < typename Range >
    auto norm_1 (Range range)
    { return detail::norm_1<Range>::apply(range); }

  } // namespace math

} // namespace sample_space

#endif
