/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef SAMPLE_SPACE_VECTOR_VECTOR_FWD_HPP
#define SAMPLE_SPACE_VECTOR_VECTOR_FWD_HPP 1

#include <limits>
#include <traits/convert.hpp>
#include <traits/domain.hpp>
#include <traits/key_type.hpp>
#include <traits/mapped_type.hpp>
#include <traits/probability.hpp>
#include <traits/real.hpp>
#include <traits/value.hpp>
#include <traits/variate.hpp>
#include <type_traits/is_map_fwd.hpp>
#include <use_default.hpp>

namespace sample_space
{

  namespace vector
  {

    /// \brief A vector space class
    /// \tparam T The type of field
    /// \tparam Index The type of index for dimensions
    /// \tparam Tag The tag defining the type of vector space
    /// \tparam Basis The basis for the vector space
    /// \tparam Storage The type of storage for the vector
    ///
    template < typename T = double
	       , typename Index = void
	       , typename Tag = use_default<>
	       , typename Basis = use_default<>
	       , typename Storage = use_default<>
	       >
    class vector;

  } // namespace vector
  
} // namespace sample_space

namespace traits
{

  namespace defaults
  {

    template < typename T
	       , typename Index
	       , typename Tag
	       , typename Basis
	       , typename Storage
	       , typename U
	       >
    struct convert< sample_space::vector::vector<T,Index,Tag,Basis,Storage>,U >
    {
      using type = sample_space::vector::vector<U,Index,Tag,Basis,Storage>;
    };
  
    template < typename T
	       , typename Index
	       , typename Tag
	       , typename Basis
	       , typename Storage
	       , typename U
	       >
    struct convert< sample_space::vector::vector<T,Index,Tag,Basis,Storage> const,U >
    {
      using type = sample_space::vector::vector<U,Index,Tag,Basis,Storage> const;
    };
  
    template < typename T
	       , typename Index
	       , typename Tag
	       , typename Basis
	       , typename Storage
	       >
    struct domain< sample_space::vector::vector<T,Index,Tag,Basis,Storage>,std::false_type >
      : domain<T>
    {};

    template < typename T
	       , typename Index
	       , typename Tag
	       , typename Basis
	       , typename Storage
	       >
    struct probability< sample_space::vector::vector<T,Index,Tag,Basis,Storage> >
      : probability<T>
    {};

    template < typename T
	       , typename Index
	       , typename Tag
	       , typename Basis
	       , typename Storage
	       >
    struct real< sample_space::vector::vector<T,Index,Tag,Basis,Storage>,std::false_type,std::false_type >
      : real<T>
    {};

    template < typename T
	       , typename Index
	       , typename Tag
	       , typename Basis
	       , typename Storage
	       >
    struct value< sample_space::vector::vector<T,Index,Tag,Basis,Storage> >
    {
      using type = T;
    };

    template < typename T, typename Index, typename Tag, typename Basis, typename Storage >
    struct variate< sample_space::vector::vector<T,Index,Tag,Basis,Storage> >
    {
      using type = sample_space::vector::vector<T,Index,Tag,Basis,Storage>;
    };

  } // namespace defaults

  template < typename T, typename Tag, typename Basis, typename Storage >
  class key_type< sample_space::vector::vector<T,void,Tag,Basis,Storage> >
  {};

  template < typename T, typename Index, typename Tag, typename Basis, typename Storage >
  struct key_type< sample_space::vector::vector<T,Index,Tag,Basis,Storage> >
  { using type = Index; };

  template < typename T, typename Tag, typename Basis, typename Storage >
  class mapped_type< sample_space::vector::vector<T,void,Tag,Basis,Storage> >
  {};

  template < typename T, typename Index, typename Tag, typename Basis, typename Storage >
  struct mapped_type< sample_space::vector::vector<T,Index,Tag,Basis,Storage> >
  { using type = T; };

} // namespace traits

namespace type_traits
{

  template < typename T, typename Tag, typename Basis, typename Storage >
  struct is_map< sample_space::vector::vector<T,void,Tag,Basis,Storage> >
    : std::false_type
  {};

  template < typename T, typename Index, typename Tag, typename Basis, typename Storage >
  struct is_map< sample_space::vector::vector<T,Index,Tag,Basis,Storage> >
    : std::true_type
  {};

} // namespace type_traits

namespace std
{

  template < typename T
	     , typename Index
	     , typename Tag
	     , typename Basis
	     , typename Storage
	     >
  struct numeric_limits< sample_space::vector::vector<T,Index,Tag,Basis,Storage> >
    : numeric_limits<T>
  {};
  
} // namespace std

#endif
