/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MODULE graph_model_vertex_trapping_eval_base

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <graph_model/vertex/trapping_eval_base.hpp>
#include <graph_model/vertex/eval_base.hpp>
#include <graph_model/vertex/exception/diagnostic_information.hpp>
#include <iostream>

#include <boost/mpl/vector.hpp>

using test_types = boost::mpl::vector< int,long,float,double >;

template < typename T >
struct S
  : graph_model::vertex::trapping_eval_base<S<T>,graph_model::vertex::eval_base<T>>
{};

BOOST_AUTO_TEST_CASE_TEMPLATE(test_default_constructor,T,test_types)
{
  S<T> s;
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_throw,T,test_types)
{
  S<T> s;

  BOOST_CHECK_THROW(s(),std::exception);
  BOOST_CHECK_THROW(s(),boost::exception);
  BOOST_CHECK_THROW(s(),std::logic_error);

  try { s(); }
  catch (std::exception const& e)
  {
    BOOST_CHECK_EQUAL(e.what(),"missing strategy");
  }
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_diagnostic_info,T,test_types)
{
  S<T> s;

  BOOST_CHECK_THROW(s(),std::exception);
  BOOST_CHECK_THROW(s(),boost::exception);
  BOOST_CHECK_THROW(s(),std::logic_error);

  try { s(); }
  catch (boost::exception const& e)
  {
    std::cout << diagnostic_information(e) << std::endl;
  }
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_uncaught_throw,T,test_types)
{
  std::cout << "XXX -- EXPECT AN ERROR" << std::endl;
  S<T> s;
  s();
}

