/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#define BOOST_TEST_MODULE sample_space_math_categorical_pdf

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include "sample_space/math/categorical_pdf.hpp"
#include "sample_space/vector/simplex.hpp"
#include "sample_space/vector/vector.hpp"
#include <cmath>
#include <iostream>
#include <vector>
#include <boost/mpl/pair.hpp>
#include <boost/mpl/vector.hpp>

using test_types = boost::mpl::vector<
  boost::mpl::pair<std::vector<float>,std::vector<int>>
  , boost::mpl::pair<std::vector<float>,std::vector<long>>
  , boost::mpl::pair<std::vector<double>,std::vector<int>>
  , boost::mpl::pair<std::vector<double>,std::vector<long>>
  , boost::mpl::pair<std::vector<double>,sample_space::vector::vector<long>>
  , boost::mpl::pair<sample_space::vector::simplex<double>,std::vector<long>>
  , boost::mpl::pair<sample_space::vector::simplex<long double>,sample_space::vector::vector<long long>>
  >;

BOOST_AUTO_TEST_CASE_TEMPLATE(test_categorical_pdf,Pair,test_types)
{
  using probability_vector_type = typename Pair::first;
  using probability_type = typename probability_vector_type::value_type;
  using size_vector_type = typename Pair::second;
  using size_type = typename size_vector_type::value_type;

  using std::abs;
  using std::pow;
  using sample_space::math::make_categorical_pdf;
  
  probability_vector_type p = { 0.1, 0.2, 0.3, 0.4 };
  size_vector_type n = { 1, 2, 3, 4 };

  auto expected
    = pow(p[0],n[0])
    * pow(p[1],n[1])
    * pow(p[2],n[2])
    * pow(p[3],n[3]);
  
  auto categorical_pdf = make_categorical_pdf(p,n);
  BOOST_CHECK_EQUAL(categorical_pdf.value(),expected);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_categorical_pdf_ratio,Pair,test_types)
{
  using probability_vector_type = typename Pair::first;
  using probability_type = typename probability_vector_type::value_type;
  using size_vector_type = typename Pair::second;
  using size_type = typename size_vector_type::value_type;

  using std::abs;
  using std::pow;
  using sample_space::math::make_categorical_pdf;
  
  probability_vector_type p_1 = { 0.1, 0.2, 0.3, 0.4 };
  size_vector_type n_1 = { 1, 2, 3, 4 };

  probability_vector_type p_2 = { 0.1, 0.2, 0.3, 0.4 };
  size_vector_type n_2 = { 4, 3, 2, 1 };

  auto expected_1
    = pow(p_1[0],n_1[0])
    * pow(p_1[1],n_1[1])
    * pow(p_1[2],n_1[2])
    * pow(p_1[3],n_1[3]);

  auto expected_2
    = pow(p_2[0],n_2[0])
    * pow(p_2[1],n_2[1])
    * pow(p_2[2],n_2[2])
    * pow(p_2[3],n_2[3]);

  auto expected = expected_1 / expected_2;
  
  auto categorical_pdf_ratio
    = make_categorical_pdf(p_1,n_1) / make_categorical_pdf(p_2,n_2);
  BOOST_CHECK_EQUAL(categorical_pdf_ratio.value(),expected);
}
